/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer *
* Copyright (C) 2021 Renesas Electronics Corporation. All rights reserved.    
*******************************************************************************/
/*******************************************************************************
* File Name    : r_hs3001.c
* Version      : 1.0
* Description  : Device driver for Renesas Sensor HS3001.
*              : (Relative humidity, Temperature)
******************************************************************************/
/*****************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 26.02.2021 1.00     First Release
******************************************************************************/

/******************************************************************************
Includes
******************************************************************************/

#include "r_cg_macrodriver.h"
#include "r_cg_serial.h"
#include "r_cg_timer.h"
#include "types.h"
#include "r_hs3001.h"
#include "r_cg_userdefine.h"

/******************************************************************************
Typedef definitions
******************************************************************************/

/******************************************************************************
Macro definitions
******************************************************************************/

#define IICA_START_CONDITION_WAIT_COUNT     100

/******************************************************************************
Exported global variables (to be accessed by other files)
******************************************************************************/

uint32_t    humidity;
uint32_t    temperature;

/* The ADC converison time. Used in r_HS3001_msleep() function.    */
/* Also used in r_cg_timer_user.c as a decrement counter. */
int16_t wait_timer_count;

/******************************************************************************
Private global variables and functions
******************************************************************************/
static void r_HS3001_msleep(int16_t msec);

MD_STATUS R_IIC_Send(uint8_t adr, uint8_t * const tx_buf, uint16_t tx_num);
MD_STATUS R_IIC_Receive(uint8_t adr, uint8_t * const rx_buf, uint16_t rx_num);
void R_IIC_Transfer_End(void);

/******************************************************************************
* Function Name: R_HS3001_Init
* Description  : Initialize variables.
*                And initialize the IICA0 varibles.
* Arguments    : void
* Return Value : void
******************************************************************************/
void R_HS3001_Init(void)
{
    /* clear tranfer complete flag */
    /* used in r_cg_serial.c and r_cg_serial_user.c*/
    g_iic_trans_comp_flag = 0;

    return;
}

/******************************************************************************
* Function Name: R_HS3001_Measurement
* Description  : Measure relative humidity and temperature
* Arguments    : void
* Return Value : HS3001_STATUS_OK - 
*                    Success
*                HS3001_STATUS_I2C_TRANSFER_ERROR - 
*                    I2C transfer error
******************************************************************************/
hs3001_status_t R_HS3001_Measurement(hs3001_sensor_data_t *sd)
{
	uint8_t     tx_buf[1] = {0};
	uint8_t     rx_buf[4] = {0};

    uint8_t     status_hs;

    MD_STATUS   ret;

    /* MR command (1 byte) = Device slave address and Write bit */
    ret = R_IIC_Send(HS3001_CMD_MR, tx_buf, 0);
    if(ret != MD_OK)
    {
        return HS3001_STATUS_I2C_TRANSFER_ERROR;
    }

    r_HS3001_msleep(HS3001_MEASUREMENT_TIME);

    /* DF command (1 byte) = Device slave address and Read bit */
    /* Measurement data (4 bytes) */
    ret = R_IIC_Receive(HS3001_CMD_DF, rx_buf, 4);
    if(ret != MD_OK)
    {
        return HS3001_STATUS_I2C_TRANSFER_ERROR;
    }

    /* Data valid or invalid */
    status_hs = ((rx_buf[0] & 0x40) == 0) ? HS3001_STATUS_OK : HS3001_STATUS_ERROR;

    if (status_hs == HS3001_STATUS_OK)
    {
        /* Humidity */
        humidity = (uint32_t)(((rx_buf[0] & 0x3F) << 8) | rx_buf[1]);   /* clear status bit */
        humidity = ((humidity * 10000) / HS3001_DENOMINATOR);

        sd->h_integer = humidity / 100;
        sd->h_decimal = humidity % 100;

        /* Temperature */
        temperature = (uint32_t)((rx_buf[2] << 8) | rx_buf[3]);
        temperature = temperature >> 2;
        temperature = (((temperature * 100) / HS3001_DENOMINATOR) * 165) - (40 * 100);

        sd->t_integer = temperature / 100;
        sd->t_decimal = temperature % 100;
    }
    else
    {
        sd->h_integer = 0;
        sd->h_decimal = 0;
        sd->t_integer = 0;
        sd->t_decimal = 0;
    }

    return HS3001_STATUS_OK;
}

/******************************************************************************
* Function Name: r_HS3001_msleep
* Description  : Wait for the ADC conversion complete.
*                Uses the TAU0 Channel0 of RL78
* Arguments    : int16_t* msec -
*                    wait time.
* Return Value : void
******************************************************************************/
static void r_HS3001_msleep(int16_t msec)
{
    wait_timer_count = msec; /* msec */

    R_TAU0_Channel0_Start();
    /* Decrement of wait_timer_count is in r_tau0_channel0_interrupt() */
    while(wait_timer_count > 0);
    R_TAU0_Channel0_Stop();
}

/******************************************************************************
* Function Name: R_IIC_Send
* Description  : Wrapper function of IIC send function.
* Arguments    : uint8_t adr -
 *                   Slave address.
*                uint8_t * const tx_buf -
*                    Pointer to send data buffer.
*                uint16_t tx_num -
*                    Number of send data.
* Return Value : MD_OK - 
*                    Success.
*                MD_ERROR1 - 
*                    At the start of transmission, the bus is busy.
*                MD_ERROR2 - 
*                    Start condition is not detected.
*                MD_ERROR3 - 
*                    Bus release was time out.
******************************************************************************/
MD_STATUS R_IIC_Send(uint8_t adr, uint8_t * const tx_buf, uint16_t tx_num)
{
    MD_STATUS   ret;

    g_iic_trans_comp_flag = IIC_TRANSFER_NOT_YET;
    ret = R_IICA0_Master_Send(adr, tx_buf, tx_num, IICA_START_CONDITION_WAIT_COUNT);
    if (ret == MD_OK)
    {
        while (g_iic_trans_comp_flag == IIC_TRANSFER_NOT_YET);
        if (g_iic_trans_comp_flag == IIC_TRANSFER_COMPLETE)
        {
            ret = MD_OK;
        }
        else
        {
            ret = MD_ERROR3;/* BUS RELEASE TIME OUT */
        }
    }
    return (ret);
}

/******************************************************************************
* Function Name: R_IIC_Receive
* Description  : Wrapper function of IIC receive function.
* Arguments    : uint8_t adr -
 *                   Slave address.
*                uint8_t * const rx_buf -
*                    Pointer to receive data buffer.
*                uint16_t rx_num -
*                    Number of receive data.
* Return Value : MD_OK - 
*                    Success.
*                MD_ERROR1 - 
*                    At the start of transmission, the bus is busy.
*                MD_ERROR2 - 
*                    Start condition is not detected.
*                MD_ERROR3 - 
*                    Bus release was time out.
******************************************************************************/
MD_STATUS R_IIC_Receive(uint8_t adr, uint8_t * const rx_buf, uint16_t rx_num)
{
    MD_STATUS   ret;

    g_iic_trans_comp_flag = IIC_TRANSFER_NOT_YET;
    ret = R_IICA0_Master_Receive(adr, rx_buf, rx_num, IICA_START_CONDITION_WAIT_COUNT);
    if (ret == MD_OK)
    {
        while (g_iic_trans_comp_flag == IIC_TRANSFER_NOT_YET);
        if (g_iic_trans_comp_flag == IIC_TRANSFER_COMPLETE)
        {
            ret = MD_OK;
        }
        else
        {
            ret = MD_ERROR3;/* BUS RELEASE TIME OUT */
        }
    }
    return (ret);
}

/******************************************************************************
* Function Name: R_IIC_Transfer_End
* Description  : Wait for end of transfer of IIC.
* Arguments    : void
* Return Value : void
******************************************************************************/
void R_IIC_Transfer_End(void)
{
    while(g_iic_trans_comp_flag == IIC_TRANSFER_NOT_YET);
    g_iic_trans_comp_flag = IIC_TRANSFER_NOT_YET;

    return;
}
