/**
 ****************************************************************************************
 *
 * @file        rble_app.c
 *
 * @brief       rBLE Sample Application
 *
 * Copyright(C) 2013-2020 Renesas Electronics Corporation
 *
 * $Rev: 6128 $
 *
 ****************************************************************************************
 */

#include    "r_cg_macrodriver.h"
#include    "timer.h"
#include    <stdio.h>
#include    <string.h>
#include    <stdlib.h>
#include    "ll.h"
#include    "rble.h"
#include    "rble_app.h"
#include    "rscip_ext.h"
#include    "prf_sel.h"
#include    "vuarts.h"
#include    "r_cg_userdefine.h"
#include    "seclib.h"

#ifndef char_t
typedef char            char_t;
#endif

/**************************** Define Declaration **************************************/
#define     RUN_COMMAND_QUEUE_SIZE          10
#define     ILLEGAL_CONNECTION_HANDLE       0xffff
#define     SEND_DATA_INTERVAL              100     // 100 * 10 = 1000ms(1s)
#define     SW_CHATTER_CNT_MAX              4

enum APP_KEY_FLAG_enum {
    APP_LTK,
    APP_ENCKEY,
    APP_IDKEY,
    APP_SIGNKEY
};

typedef struct {
    uint8_t                         Key_Set_Flags;
    struct RBLE_EVT_SM_Key_t        EncKey;
    struct RBLE_EVT_SM_Key_t        IrKey;
    struct RBLE_EVT_SM_Key_t        SignKey;
} APP_KEY_DATA;

cb_func_t   cb_func;

/******************************* PRIVATE PROTOTYPE ************************************/
static void APP_RBLE_CallBack( RBLE_MODE mode );
static BOOL APP_RBLE_Active_CallBack( void );
static BOOL APP_RBLE_Exit_CallBack( void );

/* --------------------------- Generic Access Profile --------------------------------*/
static RBLE_STATUS APP_GAP_Broadcast_Enable_Command( void );
static RBLE_STATUS APP_GAP_Set_Bonding_Mode_Command( void );
static RBLE_STATUS APP_GAP_Set_Security_Request_Command( void );
static RBLE_STATUS APP_GAP_Bonding_Response_Command( void );
static RBLE_STATUS APP_GAP_Disconnection_Command( void );

static void APP_GAP_CallBack( RBLE_GAP_EVENT *event );
static BOOL APP_GAP_Broadcast_Enable_CallBack( RBLE_GAP_EVENT *event );
static BOOL APP_GAP_Connection_CallBack( RBLE_GAP_EVENT *event );
static BOOL APP_GAP_Disconnection_CallBack( RBLE_GAP_EVENT *event );

/* ------------------------------ Security Library -----------------------------------*/
static void APP_SECLIB_Init_Command( void );
static void APP_SECLIB_CallBack(SECLIB_EVENT *event);
static void APP_SECLIB_Init_Callback( void );
uint16_t SecLib_Rand(void);

/* -------------------- Genral Purpose Communication(VUART) --------------------------*/
/* APIs. */
extern RBLE_STATUS RBLE_VUART_Server_Enable(uint16_t conhdl, RBLE_VUART_EVENT_HANDLER callback);
extern RBLE_STATUS RBLE_VUART_Server_Disable(void);
extern RBLE_STATUS RBLE_VUART_Server_Send_Indication(const char *chars, uint16_t len);
/* Call back function. */
static void APP_GPCS_CallBack(RBLE_VUART_EVENT *event);
/* Call back event functions. */
static BOOL APP_GPCS_Server_Enable_CMP(RBLE_VUART_EVENT *event);
static BOOL APP_GPCS_Server_Write_REQ(RBLE_VUART_EVENT *event);
static BOOL APP_GPCS_Server_Indication_CFM(RBLE_VUART_EVENT *event);
/* APP_Run() commands. */
static RBLE_STATUS APP_GPCS_Enable_Command(void);

/* --------------------------------- Utility -----------------------------------------*/
static void APP_Set_RunCmd( uint16_t cmd );
void rBLEAPI_Register_CB(cb_func_t *cbf);
void APP_LED1_Blink(void);
void APP_LED2_Blink(void);
#ifndef USE_SENSOR
void APP_GPCS_Send_data(void);
int8_t  app_gpcs_cfm = 0;
#endif
void APP_SW_Pushed(void);
static void APP_Delete_Bonding_Info(void);

/**************************** PUBLIC Memory Declaration *******************************/

/**************************** PRIVATE Memory Declaration ******************************/
static uint16_t         ConnectionHdl = ILLEGAL_CONNECTION_HANDLE;
static uint16_t         RunCmd[RUN_COMMAND_QUEUE_SIZE];    // RBLE_APP_CMD_NO_enum value
static uint8_t          WrPtr     ;
static uint8_t          RdPtr     ;
static uint8_t          SwChatterCnt  ;
static uint8_t          SwPushedCnt   ;

static RBLE_BROADCAST_ENABLE_PARAM  g_advdata = {
    /* General Discoverable & Undirected Connectable */
    RBLE_GAP_GEN_DISCOVERABLE,
    RBLE_GAP_UND_CONNECTABLE ,
    {
        {
            0x30,                                       /* AdvIntervalMin */
            0x60,                                       /* AdvIntervalMax */
            RBLE_GAP_ADV_CONN_UNDIR,                    /* AdvType        */
            RBLE_ADDR_PUBLIC,                           /* OwnAddrType    */
            RBLE_ADDR_PUBLIC,                           /* DirectAddrType */
            { 0xbc, 0x9a, 0x78, 0x56, 0x34, 0x12 },     /* DirectAddr     */
            RBLE_ADV_ALL_CHANNELS,                      /* AdvChannelMap  */
            RBLE_ADV_ALLOW_SCAN_ANY_CON_ANY,            /* AdvFiltPolicy  */
            0x00,                                       /* Reserved       */
        },
        {
            /* Advertising Data Length */
            3 + 13,                 /* [Flags] + [Complete local name] */

            /* Advertising Data (max 31bytes) */
            2,                      /* Number of data */
                0x01,               /* AD Types: Flags */ 
                0x06,
            12,                     /* Number of data */
                0x09,               /* AD Types: Complete local name */
                'R', 'T', 'K', '5', 'R', 'L', 'G', '1', '4', '0', 'C', /* String */

            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        },
        {
            /* Scan Rsponse Data Length */
            0,
            /* Scan Response Data (max 31bytes) */
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        }
    }
};

/* ##### Security Library Parameter ##### */
static SECLIB_PARAM app_sec_param = {
    RBLE_SLAVE,                     /* role */
    RBLE_AUTH_REQ_NO_MITM_BOND,     /* auth_req */
    RBLE_IO_CAP_NO_INPUT_NO_OUTPUT, /* iocap */
    FALSE,                          /* rpa_generate */
};

/******************************* Program Area *****************************************/

/**************************************************************************************/
/**
 *  @brief      Application Initialize
 *  
 *  @retval     TRUE if succeeded , FALSE if failed
 */
/******************************************************************************************/
BOOL APP_Init( void )
{
    BOOL        ret = TRUE;
    RBLE_STATUS ret_status;

    /* Initialize rBLE */
    ret_status = RBLE_Init(&APP_RBLE_CallBack);
    if ( RBLE_OK != ret_status ) {
        ret = FALSE;
    }
    /* Initialize Run Command Queue Pointer, LCD String Buffer */
    WrPtr = 0;
    RdPtr = 0;

    /* Initialize SW_USR count */
    SwChatterCnt = 0;
    SwPushedCnt  = 0;

    return( ret );
}

/**************************************************************************************/
/**
 *  @brief      Application Command Execute
 *  
 *  @retval     None
 */
/******************************************************************************************/
void APP_Run( void )
{
    /* Loop until Run Command Queue is empty */
    while(RdPtr != WrPtr)
    {
        switch(RunCmd[RdPtr])
        {
            case RBLE_APP_CMD_NON:
                break;
            /* ---- Security Library ----*/
            case SECLIB_INIT_CMD:
                APP_SECLIB_Init_Command();
                break;
            case SECLIB_SET_PARAM_CMD:
                break;
            /* --------------------------- Generic Access Profile --------------------------------*/
            case GAP_BROADCAST_ENABLE_CMD:
                APP_GAP_Broadcast_Enable_Command();
                break;
            case GAP_SET_BONDING_MODE_CMD:
                APP_GAP_Set_Bonding_Mode_Command();
                break;
            case GAP_SET_SECURITY_REQUEST_CMD:
                APP_GAP_Set_Security_Request_Command();
                break;
            case GAP_BONDING_RESPONSE_CMD:
                APP_GAP_Bonding_Response_Command();
                break;
            case GAP_DISCONNECT_CMD:
                APP_GAP_Disconnection_Command();
                break;
            /* ------------------- Genral Purpose Communication(VUART) --------------------------*/
            case GPCS_Server_Enable_CMD:
                APP_GPCS_Enable_Command();
                break;
            case GPCS_Server_Disable_CMD:
                break;
            case GPCS_Server_Send_Indication_CMD:
                break;
            default:
                break;
        }
        
        /* Operate Run Command Queue Pointer as a Ring Buffer */
        GLOBAL_INT_DISABLE();
        RdPtr = (RdPtr == (RUN_COMMAND_QUEUE_SIZE - 1)) ? (0) : (RdPtr + 1);
        GLOBAL_INT_RESTORE();
    }
}

/**************************************************************************************/
/**
 *  @brief      Security Library API Call : SecLib Init
 *  
 *  @retval     None
 */
/******************************************************************************************/
static void APP_SECLIB_Init_Command( void )
{
    ConnectionHdl = ILLEGAL_CONNECTION_HANDLE;

    SecLib_Init( &APP_GAP_CallBack, NULL, &APP_SECLIB_CallBack );
}

/**************************************************************************************/
/**
 *  @brief      rBLE API Call : GAP Broadcast Enable
 *  
 *  @retval     rBLE API Return Status
 */
/******************************************************************************************/
static RBLE_STATUS APP_GAP_Broadcast_Enable_Command( void )
{
    RBLE_STATUS ret_status;

    ret_status = RBLE_GAP_Broadcast_Enable( g_advdata.disc_mode, g_advdata.conn_mode, &g_advdata.adv_info );

    return( ret_status );
}

/**************************************************************************************/
/**
 *  @brief      rBLE API Call : GAP Set Bonding Mode
 *  
 *  @retval     rBLE API Return Status
 */
/******************************************************************************************/
static RBLE_STATUS APP_GAP_Set_Bonding_Mode_Command( void )
{
    RBLE_STATUS ret_status;

    ret_status = RBLE_GAP_Set_Bonding_Mode(RBLE_GAP_BONDABLE);

    return( ret_status );
}

/**************************************************************************************/
/**
 *  @brief      rBLE API Call : GAP Set Security Request
 *  
 *  @retval     rBLE API Return Status
 */
/******************************************************************************************/
static RBLE_STATUS APP_GAP_Set_Security_Request_Command( void )
{
    RBLE_STATUS ret_status;

    ret_status = RBLE_GAP_Set_Security_Request( RBLE_GAP_SEC1_NOAUTH_PAIR_ENC );

    return( ret_status );
}

/**************************************************************************************/
/**
 *  @brief      rBLE API Call : GAP Bonding Response
 *  
 *  @retval     rBLE API Return Status
 */
/******************************************************************************************/
static RBLE_STATUS APP_GAP_Bonding_Response_Command( void )
{
    RBLE_STATUS ret_status;
    static RBLE_BOND_RESP_PARAM Param = {
        0x0000,                                         /* Connection handle           */
        RBLE_OK,                                        /* accept or reject bonding    */
        RBLE_IO_CAP_NO_INPUT_NO_OUTPUT,                 /* IO capabilities             */
        RBLE_OOB_AUTH_DATA_NOT_PRESENT,                 /* OOB flag                    */
        RBLE_AUTH_REQ_NO_MITM_BOND,                     /* Authentication Requirements */
        RBLE_SMP_MAX_ENC_SIZE_LEN,                      /* Encryption key size         */
        RBLE_KEY_DIST_ENCKEY | RBLE_KEY_DIST_IDKEY,     /* Initiator key distribution  */
        RBLE_KEY_DIST_ENCKEY,                           /* Responder key distribution  */
        0x00                                            /* Reserved                    */
    };

    Param.conhdl = ConnectionHdl;
    ret_status = RBLE_GAP_Bonding_Response( &Param );

    return( ret_status );
}

/**************************************************************************************/
/**
 *  @brief      rBLE API Call : GAP Disconnection
 *  
 *  @retval     rBLE API Return Status
 */
/******************************************************************************************/
static RBLE_STATUS APP_GAP_Disconnection_Command( void )
{
    RBLE_STATUS ret_status = FALSE;

    if(ConnectionHdl != ILLEGAL_CONNECTION_HANDLE)
    {
        ret_status = RBLE_GAP_Disconnect( ConnectionHdl );
    }

    return( ret_status );
}

/**************************************************************************************/
/**
 *  @brief      rBLE API Call : Genral purpose communication(VUART) server enable
 *  
 *  @retval     rBLE API Return Status
 */
/******************************************************************************************/
static RBLE_STATUS APP_GPCS_Enable_Command(void)
{
    RBLE_STATUS ret_status;

    ret_status = RBLE_VUART_Server_Enable(ConnectionHdl, &APP_GPCS_CallBack);

    return(ret_status);
}

/**************************************************************************************/
/**
 *  @brief      Application Idle Check
 *  
 *  @retval     TRUE if Command Queue is empty , FALSE if it not empty
 */
/******************************************************************************************/
BOOL APP_Is_Idle( void )
{
    BOOL    Idle = FALSE;

    /* Application Status is Idle when Command Queue is Empty */
    if(RdPtr == WrPtr)
    {
        Idle = TRUE;
    }

    return( Idle );
}

/********************  Call Back API ********************/

/******************************************************************************************/
/**
 *  @brief      Event Handle : rBLE
 *  
 *  @param      mode    rBLE Mode , defined by RBLE_MODE_enum
 *
 *  @retval     None
 */
/******************************************************************************************/
static void APP_RBLE_CallBack( RBLE_MODE mode )
{
    switch(mode){
        case RBLE_MODE_INITIALIZE:
            break;
        case RBLE_MODE_ACTIVE:
            APP_RBLE_Active_CallBack();
            break;
        case RBLE_MODE_RESET:
            break;
        case RBLE_MODE_SLEEP:
            break;
        case RBLE_MODE_ERROR:
            APP_RBLE_Exit_CallBack();
            break;
        default:
            break;
    }
}

/******************************************************************************************/
/**
 *  @brief      Event Respond : rBLE Active Event
 *  
 *  @retval     TRUE if succeeded , FALSE if failed
 */
/******************************************************************************************/
static BOOL APP_RBLE_Active_CallBack( void )
{
    APP_Set_RunCmd(SECLIB_INIT_CMD);

    return( TRUE );
}

/******************************************************************************************/
/**
 *  @brief      Event Respond : rBLE Exit Event
 *  
 *  @retval     TRUE if succeeded , FALSE if failed
 */
/******************************************************************************************/
static BOOL APP_RBLE_Exit_CallBack( void )
{
    return( TRUE );
}

/* --------------------------- Generic Access Profile --------------------------------*/

/******************************************************************************************/
/**
 *  @brief      Event Handler : GAP Event
 *
 *  @param      event   GAP Event Parameters
 *
 *  @retval     None
 */
/******************************************************************************************/
static void APP_GAP_CallBack( RBLE_GAP_EVENT *event )
{
    switch( event->type ) {
        case RBLE_GAP_EVENT_SET_NAME_COMP:
            break;
        case RBLE_GAP_EVENT_OBSERVATION_ENABLE_COMP:
            break;
        case RBLE_GAP_EVENT_OBSERVATION_DISABLE_COMP:
            break;
        case RBLE_GAP_EVENT_BROADCAST_ENABLE_COMP:
            APP_GAP_Broadcast_Enable_CallBack( event );

            /* LED1 Blink */
            if(RBLE_OK == event->param.status) {
                RBLE_Set_Timer_Id(_RBLE_TIMER_ID_LED, (RBLE_FUNC)APP_LED1_Blink, LED_BLINK_INTERVAL);
            }
            break;
        case RBLE_GAP_EVENT_BROADCAST_DISABLE_COMP:
            break;
        case RBLE_GAP_EVENT_GET_DEVICE_INFO_COMP:
            break;
        case RBLE_GAP_EVENT_GET_WHITE_LIST_SIZE_COMP:
            break;
        case RBLE_GAP_EVENT_ADD_TO_WHITE_LIST_COMP:
            break;
        case RBLE_GAP_EVENT_DEL_FROM_WHITE_LIST_COMP:
            break;
        case RBLE_GAP_EVENT_GET_REMOTE_DEVICE_NAME_COMP:
            break;
        case RBLE_GAP_EVENT_GET_REMOTE_DEVICE_INFO_COMP:
            break;
        case RBLE_GAP_EVENT_DEVICE_SEARCH_COMP:
            break;
        case RBLE_GAP_EVENT_DEVICE_SEARCH_RESULT_IND:
            break;
        case RBLE_GAP_EVENT_SET_RANDOM_ADDRESS_COMP:
            break;
        case RBLE_GAP_EVENT_CONNECTION_COMP:
            APP_GAP_Connection_CallBack( event );

            /* LED1 on */
            if(RBLE_OK == event->param.status) {
                RBLE_Clear_Timer_Id(_RBLE_TIMER_ID_LED);
                LED1 = LED_ON;
            }
            break;
        case RBLE_GAP_EVENT_CONNECTION_CANCEL_COMP:
            break;
        case RBLE_GAP_EVENT_DISCONNECT_COMP:
            APP_GAP_Disconnection_CallBack( event );

            /* LED1 and LED2 off */
            RBLE_Clear_Timer_Id(_RBLE_TIMER_ID_LED);
            LED1 = LED_OFF;
            LED2 = LED_OFF;
            break;
        case RBLE_GAP_EVENT_ADVERTISING_REPORT_IND:
            break;
        case RBLE_GAP_EVENT_CHANGE_CONNECTION_PARAM_REQ_IND:
            break;
        case RBLE_GAP_EVENT_CHANGE_CONNECTION_PARAM_COMP:
            break;
        case RBLE_GAP_EVENT_CHANGE_CONNECTION_PARAM_RESPONSE:
            break;
        case RBLE_GAP_EVENT_CHANNEL_MAP_REQ_COMP:
            break;
        case RBLE_GAP_EVENT_READ_RSSI_COMP:
            break;
        case RBLE_GAP_EVENT_WR_CHAR_IND:
            break;
        case RBLE_GAP_EVENT_COMMAND_DISALLOWED_IND:
            break;
        default:
            break;
    }
}

/******************************************************************************************/
/**
 *  @brief      Event Respond : GAP Broadcast Enable Complete Event
 *
 *  @param      event   GAP Event Parameters
 *
 *  @retval     TRUE if succeeded , FALSE if failed
 */
/******************************************************************************************/
static BOOL APP_GAP_Broadcast_Enable_CallBack( RBLE_GAP_EVENT *event )
{
    RBLE_STATUS     ret_status = FALSE;

    if(RBLE_OK == event->param.status) {
        ret_status = TRUE;
    }

    return( ret_status );
}

/******************************************************************************************/
/**
 *  @brief      Event Respond : GAP Connection Complete Event
 *
 *  @param      event   GAP Event Parameters
 *
 *  @retval     TRUE if succeeded , FALSE if failed
 */
/******************************************************************************************/
static BOOL APP_GAP_Connection_CallBack( RBLE_GAP_EVENT *event )
{
    RBLE_STATUS ret_status = FALSE;

    if(RBLE_OK == event->param.status) {
        ConnectionHdl  = event->param.conn_comp.connect_info.conhdl;
        ret_status = TRUE;
        
        /* Enable Profile (Profile is Disabled automatically when Disconnection) */
        APP_Set_RunCmd(GPCS_Server_Enable_CMD);
    }

    return( ret_status );
}

/******************************************************************************************/
/**
 *  @brief      Event Respond : GAP Disconnection Complete Event
 *
 *  @param      event   GAP Event Parameters
 *
 *  @retval     TRUE if succeeded , FALSE if failed
 */
/******************************************************************************************/
static BOOL APP_GAP_Disconnection_CallBack( RBLE_GAP_EVENT *event )
{
    ConnectionHdl = ILLEGAL_CONNECTION_HANDLE;

    /* Disable timer for indication trigger */
    RBLE_Clear_Timer_Id( _RBLE_TIMER_ID_APP );

    RBLE_VUART_Server_Disable();

    /* Restart Broadcast for the Next Connection */
    APP_Set_RunCmd(GAP_BROADCAST_ENABLE_CMD);

    return( TRUE );
}

/* ##### SecLib Handler ##### */
uint16_t SecLib_Rand(void)
{
    return rand();
}

/* ##### SecLib Event Handler ##### */
static void APP_SECLIB_CallBack(SECLIB_EVENT *event)
{
    switch (event->type) {
    case SECLIB_EVENT_INIT_COMP:
        APP_SECLIB_Init_Callback();
        break;

    case SECLIB_EVENT_SET_PARAM_COMP:
        APP_GAP_Broadcast_Enable_Command();

        break;

    case SECLIB_EVENT_PAIRING_REQ:
        SecLib_Pairing_Req_Resp(event->param.pairing_req.conhdl, TRUE);
        break;

    case SECLIB_EVENT_PASSKEY_IND:
        break;

    case SECLIB_EVENT_ENC_COMP:
        if (event->param.enc.status == RBLE_OK) {
            /* enc enabled */
        }
        else {
            /* enc failed */
        }
        break;

    case SECLIB_EVENT_PAIRING_COMP:
        if (event->param.pairing.status == RBLE_OK) {
            /* pairing completed */
        }
        else {
            /* pairing failed */
        }
        break;

    case SECLIB_EVENT_CHK_ADDR_COMP:
        if (event->param.chk_addr.status == RBLE_OK) {
            /* Bond found */
        }
        else {
            /* No bond found */
        }
        break;

    case SECLIB_EVENT_DELETE_BONDING_INFO_COMP:
        if (event->param.status == RBLE_OK) {
            /* bonding info deleted */
        }
        else {
            /* failed */
        }
        break;

    default:
        break;
    }
}

/**************************************************************************************/
/**
 *  @brief      Security Library API Call : SecLib Set Param
 *  
 *  @retval     None
 */
/******************************************************************************************/
static void APP_SECLIB_Init_Callback( void )
{
    SecLib_Set_Param(&app_sec_param);
}

/* ------------------------- General Purpose Communicaiton -------------------------------*/
static void APP_GPCS_CallBack( RBLE_VUART_EVENT *event )
{
    switch(event->type)
    {
        case RBLE_VUART_EVENT_SERVER_ENABLE_CMP:
            APP_GPCS_Server_Enable_CMP(event);
            break;
        case RBLE_VUART_EVENT_SERVER_WRITE_REQ:
            APP_GPCS_Server_Write_REQ(event);
            break;
        case RBLE_VUART_EVENT_SERVER_INDICATION_CFM:
            APP_GPCS_Server_Indication_CFM(event);
            break;
        default:
            break;
    }

    return;
}

static BOOL APP_GPCS_Server_Enable_CMP(RBLE_VUART_EVENT *event)
{
#ifdef USE_SENSOR
    /* do nothing */
#else
    app_gpcs_cfm = 0;
    RBLE_Set_Timer_Id(_RBLE_TIMER_ID_APP, APP_GPCS_Send_data, 100);
    RBLE_Set_Timer_Id(_RBLE_TIMER_ID_LED, (RBLE_FUNC)APP_LED2_Blink, LED_BLINK_INTERVAL);
#endif

    return( TRUE );
}

static BOOL APP_GPCS_Server_Write_REQ(RBLE_VUART_EVENT *event)
{
#ifdef USE_SENSOR
    cb_func.cb_receive_data((uint8_t *)event->param.server_write_req.value,
                            event->param.server_write_req.len);
#else
    /* do nothing */
#endif

    return( TRUE );
}
static BOOL APP_GPCS_Server_Indication_CFM(RBLE_VUART_EVENT *event)
{
#ifdef USE_SENSOR
    cb_func.cb_confirmation();
#else
    app_gpcs_cfm = 0;

    /* Restart the rBLE Timer. */
    RBLE_Set_Timer_Id(_RBLE_TIMER_ID_APP, APP_GPCS_Send_data, 100);
#endif

    return( TRUE );
}

/* --------------------------- Utility -------------------------------------*/

/******************************************************************************************/
/**
 *  @brief      Queueing Command
 *
 *  @param      cmd     Command Value , defined by RBLE_APP_CMD_NO_enum
 *
 *  @retval     None
 */
/******************************************************************************************/
static void APP_Set_RunCmd( uint16_t cmd )
{
    RunCmd[WrPtr] = cmd;

    /* Operate Run Command Queue Pointer as a Ring Buffer */
    GLOBAL_INT_DISABLE();
    WrPtr = (WrPtr == (RUN_COMMAND_QUEUE_SIZE - 1)) ? (0) : (WrPtr + 1);
    GLOBAL_INT_RESTORE();
}

/******************************************************************************************
 *  @brief      Register callback function.
 *  @param      void
 *  @retval     void
******************************************************************************************/
void rBLEAPI_Register_CB(cb_func_t *cbf)
{
    cb_func.cb_confirmation = cbf->cb_confirmation;
    cb_func.cb_receive_data = cbf->cb_receive_data;
    
    return;
}

/******************************************************************************************
 *  @brief      LED1 blink
 *  @param      void
 *  @retval     void
******************************************************************************************/
void APP_LED1_Blink(void)
{
    RBLE_Set_Timer_Id(_RBLE_TIMER_ID_LED, (RBLE_FUNC)APP_LED1_Blink, LED_BLINK_INTERVAL);

    LED1 ^= 1;

    return;
}

/******************************************************************************************
 *  @brief      LED2 blink
 *  @param      void
 *  @retval     void
******************************************************************************************/
void APP_LED2_Blink(void)
{
    RBLE_Set_Timer_Id(_RBLE_TIMER_ID_LED, (RBLE_FUNC)APP_LED2_Blink, LED_BLINK_INTERVAL);

    LED2 ^= 1;

    return;
}

#ifndef USE_SENSOR
/******************************************************************************************
 *  @brief      LED2 blink
 *  @param      void
 *  @retval     void
******************************************************************************************/
void APP_GPCS_Send_data(void)
{
    static uint8_t c = 'A';

    if (app_gpcs_cfm == 0)
    {
        app_gpcs_cfm = 1; /* Confirmation reception is required. */

        /* Send data by GPCS(VUART) */
        RBLE_VUART_Server_Send_Indication((const char *)&c, 1);
        if (c++ >= 'z')
        {
            c = 'A';
        }
    }

    return;
}
#endif

/******************************************************************************************
 *  @brief      The user switch (SW_USR) interrupt (INTP0) routine.
 *              Start chattering timer for SW_USER.
 *  @param      void
 *  @retval     void
******************************************************************************************/
void APP_SW_Pushed(void)
{
    if (SwChatterCnt == 0)
    {
        SwChatterCnt++;
        RBLE_Set_Timer_Id(_RBLE_TIMER_ID_SW, APP_Delete_Bonding_Info, 5);
    }
}

/******************************************************************************************
 *  @brief      The user switch (SW_USR) process.
 *              Delete bonding Information.
 *  @param      void
 *  @retval     void
******************************************************************************************/
void APP_Delete_Bonding_Info(void)
{
    /* If FPB_PORT_SW(P137/INTP0) is 0 consecutively, */
    /* the bonding information is deleted.            */

    if (FPB_PORT_SW == 0)
    {
        SwPushedCnt++;
    }
    
    if (SwChatterCnt < SW_CHATTER_CNT_MAX)
    {
        SwChatterCnt++;
        RBLE_Set_Timer_Id(_RBLE_TIMER_ID_SW, APP_Delete_Bonding_Info, 5);
    } else {

        if (SwPushedCnt == SW_CHATTER_CNT_MAX)
        {
            SecLib_Delete_Bonding_Info(SECLIB_DELETE_ALL_BONDS);
        }
        SwChatterCnt = 0;
        SwPushedCnt  = 0;
    }
}
