/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized. This 
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer *
* Copyright (C) 2021 Renesas Electronics Corporation. All rights reserved.    
*******************************************************************************/
/*******************************************************************************
* File Name    : r_hs3001_app.c
* Version      : 1.0
* Description  : Device driver for Renesas Sensor HS3001.
*              : (Relative humidity, Temperature)
******************************************************************************/
/*****************************************************************************
* History : DD.MM.YYYY Version  Description
*         : 26.02.2021 1.00     First Release
******************************************************************************/

/******************************************************************************
Includes
******************************************************************************/

#include    "r_cg_macrodriver.h"
#include    "timer.h"
#include    "ll.h"
#include    "rble.h"
#include    "vuarts.h"
#include    "r_hs3001_app.h"
#include    "r_hs3001.h"
#include    "r_cg_userdefine.h"

/******************************************************************************
Macro definitions
******************************************************************************/

#define     RUN_COMMAND_QUEUE_SIZE          10

#define     HS3001_MEASUREMENT_INTERVAL     500 /* 500 * 10ms = 5s */

/******************************************************************************
Typedef definitions
******************************************************************************/

/* HS3001 Application Run Sequence Commands */
enum HS3001_APP_CMD_NO_enum {
    HS3001_APP_CMD_NON,
    HS3001_APP_CMD_START_MEASUREMENT,
    HS3001_APP_CMD_SEND_DATA
};

/* HS3001 Application Processing State */
enum HS3001_APP_STATE_NO_enum {
    HS3001_APP_STATE_IDLE,
    HS3001_APP_STATE_MEASURING,
};

/* General Purpose Communication Server Data Structure */
typedef struct GPCS_data_st {
    uint8_t rbuf[VUART_SEND_BUFFER_SIZE];   /* Receive data buffer of BLE write request. */
    uint8_t sbuf[VUART_SEND_BUFFER_SIZE];   /* Send data buffer of BLE indication. */
    uint8_t rlen;                           /* Receive data length. */
    uint8_t slen;                           /* Send data length. */
    uint8_t indication_cfm;                 /* Indication confirmation flag. */
} GPCS_data_t;

/******************************************************************************
Exported global variables (to be accessed by other files)
******************************************************************************/

/******************************************************************************
Private global variables and functions
******************************************************************************/

/* Sensor measurement variables */
float Temperature;
float Pressure;
float Relative_Humidity;

/* Command buffer variables */
static uint16_t RunCmd[RUN_COMMAND_QUEUE_SIZE];
static uint8_t  WrPtr;
static uint8_t  RdPtr;

/* General Purpose Communication Server Data Structure */
GPCS_data_t  GPCS_data;

void R_HS3001_APP_Measurement(void);
void R_HS3001_APP_Send_Data(void);
void R_HS3001_APP_Confirmation_Callback(void);
void R_HS3001_APP_Receive_Data_Callback(uint8_t *rbuf, uint8_t len);
static void r_HS3001_APP_Set_RunCmd(uint16_t cmd);
static void r_HS3001_APP_Error(void);
extern void rBLEAPI_Register_CB(cb_func_t *cbf);
extern void APP_LED1_Blink(void);
extern void APP_LED2_Blink(void);

/******************************************************************************
* Function Name: R_HS3001_APP_Init
* Description  : Initialize the sensor application.
* Arguments    : void 
* Return Value : void
******************************************************************************/
void R_HS3001_APP_Init(void)
{
    /* Call back functions structure for data send and data receive */
    cb_func_t   cbf;

    R_HS3001_Init();

    memset(&RunCmd, 0, sizeof(RunCmd));
    WrPtr = 0;
    RdPtr = 0;

    /* Register callback functions of the receive data and the confirmation */
    cbf.cb_confirmation = R_HS3001_APP_Confirmation_Callback;
    cbf.cb_receive_data = R_HS3001_APP_Receive_Data_Callback;

    /* Register the callback functions of the sensor to rBLEAPI_app */
    rBLEAPI_Register_CB(&cbf);

    r_HS3001_APP_Set_RunCmd(HS3001_APP_CMD_NON);

    return;
}

/******************************************************************************
* Function Name: R_HS3001_APP_Run
* Description  : The sensor application controll function.
* Arguments    : void
* Return Value : void
******************************************************************************/
void R_HS3001_APP_Run(void)
{
    /* Loop until Run Command Queue is empty */
    while(RdPtr != WrPtr)
    {
        switch(RunCmd[RdPtr])
        {
            case HS3001_APP_CMD_NON:
                break;

            case HS3001_APP_CMD_START_MEASUREMENT:
                /* Start the sensor measurement */
                RBLE_Set_Timer_Id(_RBLE_TIMER_ID_APP, (RBLE_FUNC)R_HS3001_APP_Measurement,
                                  HS3001_MEASUREMENT_INTERVAL);

                /* Start LED2 Blinking */
                RBLE_Set_Timer_Id(_RBLE_TIMER_ID_LED, (RBLE_FUNC)APP_LED2_Blink,
                                  LED_BLINK_INTERVAL);

                r_HS3001_APP_Set_RunCmd(HS3001_APP_CMD_NON);
                break;

            case HS3001_APP_CMD_SEND_DATA:
                R_HS3001_APP_Send_Data();
                break;

            default:
                break;
        }
        
        /* Operate Run Command Queue Pointer as a Ring Buffer */
        GLOBAL_INT_DISABLE();
        RdPtr = (RdPtr == (RUN_COMMAND_QUEUE_SIZE - 1)) ? (0) : (RdPtr + 1);
        GLOBAL_INT_RESTORE();
    }

    return;
}

/******************************************************************************
* Function Name: R_HS3001_APP_Measurement
* Description  : Measure relative humidity and temperature.
* Arguments    : void
* Return Value : void
******************************************************************************/
void R_HS3001_APP_Measurement(void)
{
    hs3001_status_t         err;
    hs3001_sensor_data_t    sd;

    if (GPCS_data.indication_cfm == 0)
    {
        /* Measure trelative humidity and temperature. */
        err = R_HS3001_Measurement(&sd);
        if(err != HS3001_STATUS_OK)
        {
            r_HS3001_APP_Error();
        }

        /* Store the measurement data to the send buffer  */
        /* of the general purpose communication service. */
        sprintf((char *)GPCS_data.sbuf, "h%02d.%02d, t%02d.%02d", sd.h_integer, sd.h_decimal, 
                                                          sd.t_integer, sd.t_decimal);
        GPCS_data.slen = strlen((char *)GPCS_data.sbuf);

        r_HS3001_APP_Set_RunCmd(HS3001_APP_CMD_SEND_DATA);
    }

    return;
}

/******************************************************************************
* Function Name: R_HS3001_APP_Send_Data
* Description  : Send the sensor data.
* Arguments    : void
* Return Value : void
******************************************************************************/
void R_HS3001_APP_Send_Data(void)
{
    RBLE_STATUS status;

    GPCS_data.indication_cfm = 1; /* Confirmation reception is required. */

    /* Send data by GPCS(VUART) */
    status = RBLE_VUART_Server_Send_Indication((const char *)&GPCS_data.sbuf[0], GPCS_data.slen);

    if (status != RBLE_OK)
    {
        /* Stop the rBLE Timer */
        RBLE_Clear_Timer_Id(_RBLE_TIMER_ID_APP);
        RBLE_Clear_Timer_Id(_RBLE_TIMER_ID_LED);
        LED2 = LED_OFF;

        if (status != RBLE_GATT_INDICATE_NOT_ALLOWED)
        {
            r_HS3001_APP_Error();
        }
    }

    r_HS3001_APP_Set_RunCmd(HS3001_APP_CMD_NON);

    return;
}

/******************************************************************************
* Function Name: R_HS3001_APP_Confirmation_Callback
* Description  : Confirmation of indication function.
* Arguments    : void
* Return Value : void
******************************************************************************/
void R_HS3001_APP_Confirmation_Callback(void)
{
    GPCS_data.indication_cfm = 0;

    /* Restart the rBLE Timer. Call the R_HS3001_APP_Measurement() repeatedly. */
    RBLE_Set_Timer_Id(_RBLE_TIMER_ID_APP, R_HS3001_APP_Measurement, HS3001_MEASUREMENT_INTERVAL);

    return;
}

/******************************************************************************
* Function Name: R_HS3001_APP_Receive_Data_Callback
* Description  : Receive data from a smartphone(GPCS Client).
*                Called from the RBLE_VUART_EVENT_SERVER_WRITE_REQ event
*                of the GPCS server role.
* Arguments    : *rbuf - 
*                    Pointer of the receive data.
*                len - 
*                    Length of the receive data
* Return Value : void
******************************************************************************/
void R_HS3001_APP_Receive_Data_Callback(uint8_t *rbuf, uint8_t len)
{
    memcpy(&GPCS_data.rbuf[0], rbuf, len);
    GPCS_data.rlen = len;

    if ((GPCS_data.rbuf[0] == 's') || (GPCS_data.rbuf[0] == 'S'))
    {
        GPCS_data.indication_cfm = 0;

        /* Start the sensor measurement */
        r_HS3001_APP_Set_RunCmd(HS3001_APP_CMD_START_MEASUREMENT);
    }
    else
    {
        /* Receive other data. */

        /* Stop the sensor measurement */
        r_HS3001_APP_Set_RunCmd(HS3001_APP_CMD_NON);

        RBLE_Clear_Timer_Id(_RBLE_TIMER_ID_APP);

        /* LED2 OFF */
        RBLE_Clear_Timer_Id(_RBLE_TIMER_ID_LED);
        LED2 = LED_OFF;
    }

    return;
}

/******************************************************************************
* Function Name: r_HS3001_APP_Set_RunCmd
* Description  : Set application processing command to be executed next
* Arguments    : uint16_t cmd -
*                    Processing command.
* Return Value : void
******************************************************************************/
static void r_HS3001_APP_Set_RunCmd(uint16_t cmd)
{
    RunCmd[WrPtr] = cmd;

    /* Operate Run Command Queue Pointer as a Ring Buffer */
    GLOBAL_INT_DISABLE();
    WrPtr = (WrPtr == (RUN_COMMAND_QUEUE_SIZE - 1)) ? (0) : (WrPtr + 1);
    GLOBAL_INT_RESTORE();

    return;
}

/******************************************************************************
* Function Name: r_HS3001_APP_Error
* Description  : Application error function. Blinking LED and stop processing.
* Arguments    : void
* Return Value : void
******************************************************************************/
static void r_HS3001_APP_Error(void)
{
    uint32_t    i;

    DI();

    LED1 = LED_ON;
    LED2 = LED_OFF;
    while(1)
    {
        LED1 ^= 1;
        LED2 ^= 1;
        i = 200000;
        while(i--);
    }
}
